@@ -65,53 +65,7 @@ ore.search <- function (regex, text, all = FALSE, start = 1L, simplify = TRUE)
     if (!is.ore(regex))
         regex <- ore(regex, encoding=.getEncoding(text))
     
-    if (length(text) < 1)
-        stop("The text vector is empty")
-    else if (length(text) > 1)
-    {
-        start <- rep(start, length.out=length(text))
-        match <- lapply(seq_along(text), function(i) ore.search(regex, text=text[i], all=all, start=start[i]))
-    }
-    else
-    {
-        result <- .Call("ore_search", attr(regex,".compiled"), text, as.logical(all), as.integer(start), PACKAGE="ore")
-        
-        if (is.null(result))
-            match <- NULL
-        else
-        {
-            nMatches <- result[[1]]
-            indices <- seq_len(nMatches * (attr(regex,"nGroups") + 1))
-            offsets <- t(matrix(result[[2]][indices], ncol=nMatches))
-            byteOffsets <- t(matrix(result[[3]][indices], ncol=nMatches))
-            lengths <- t(matrix(result[[4]][indices], ncol=nMatches))
-            byteLengths <- t(matrix(result[[5]][indices], ncol=nMatches))
-            matchdata <- t(matrix(result[[6]][indices], ncol=nMatches))
-            
-            match <- structure(list(text=text, nMatches=nMatches, offsets=offsets[,1], byteOffsets=byteOffsets[,1], lengths=lengths[,1], byteLengths=byteLengths[,1], matches=matchdata[,1]), class="orematch")
-            
-            sourceEncoding <- .getEncoding(text)
-            Encoding(match$matches) <- sourceEncoding
-            
-            if (attr(regex, "nGroups") > 0)
-            {
-                match$groups <- list(offsets=offsets[,-1,drop=FALSE], byteOffsets=byteOffsets[,-1,drop=FALSE], lengths=lengths[,-1,drop=FALSE], byteLengths=byteLengths[,-1,drop=FALSE], matches=matchdata[,-1,drop=FALSE])
-                if (!is.null(attr(regex, "groupNames")))
-                {
-                    groupNames <- attr(regex, "groupNames")
-                    colnames(match$groups$offsets) <- groupNames
-                    colnames(match$groups$byteOffsets) <- groupNames
-                    colnames(match$groups$lengths) <- groupNames
-                    colnames(match$groups$byteLengths) <- groupNames
-                    colnames(match$groups$matches) <- groupNames
-                }
-                Encoding(match$groups$matches) <- sourceEncoding
-            }
-        }
-        
-        if (!simplify)
-            match <- list(match)
-    }
+    match <- .Call("ore_search_all", attr(regex,".compiled"), text, as.logical(all), as.integer(start), as.logical(simplify), attr(regex,"groupNames"), PACKAGE="ore")
     
     .Workspace$lastMatch <- match
     return (match)
@@ -185,7 +139,7 @@ print.orematch <- function (x, ...)
 #' These functions extract entire matches, or just subgroup matches, from
 #' objects of class \code{"orematch"}. They can also be applied to lists of
 #' these objects, as returned by \code{\link{ore.search}} when more than one
-#' string is searched.
+#' string is searched. For other objects they return \code{NA}.
 #' 
 #' @param object An R object. Methods are provided for generic lists and
 #'   \code{"orematch"} objects.
@@ -213,6 +167,13 @@ matches.orematch <- function (object, ...)
     return (object$matches)
 }
 
+#' @rdname matches
+#' @export
+matches.default <- function (object, ...)
+{
+    return (NA_character_)
+}
+
 #' @rdname matches
 #' @export
 groups <- function (object, ...)
@@ -234,6 +195,13 @@ groups.orematch <- function (object, ...)
     return (object$groups$matches)
 }
 
+#' @rdname matches
+#' @export
+groups.default <- function (object, ...)
+{
+    return (NA_character_)
+}
+
 #' Retrieve the last match
 #' 
 #' This function can be used to obtain the \code{"orematch"} object, or list,
@@ -405,9 +373,9 @@ ore.subst <- function (regex, replacement, text, all = FALSE, ...)
             {
                 currentReplacements <- rep(replacement, length.out=currentMatch$nMatches)
                 if (!is.null(groupNumberMatch))
-                    currentReplacements <- apply(currentMatch$groups$matches[as.integer(groupNumberMatch$groups$matches),,drop=FALSE], 2, function(x) do.subst(groupNumberMatch,x,replacement))
+                    currentReplacements <- apply(currentMatch$groups$matches[,as.integer(groupNumberMatch$groups$matches),drop=FALSE], 1, function(x) do.subst(groupNumberMatch,x,replacement))
                 if (!is.null(groupNameMatch))
-                    currentReplacements <- apply(currentMatch$groups$matches[groupNameMatch$groups$matches,,drop=FALSE], 2, function(x) do.subst(groupNameMatch,x,replacement))
+                    currentReplacements <- apply(currentMatch$groups$matches[,groupNameMatch$groups$matches,drop=FALSE], 1, function(x) do.subst(groupNameMatch,x,replacement))
             }
             
             result[i] <- do.subst(currentMatch, currentReplacements, text[i])
